import os
import sys
import datetime
import json

import eccodes as ecc

def keys_to_ignore():
    """Keys that the script must ignore to avoid issue with numpy arrays
    """
    return [
        'latLonValues',
        'values',
        'latitudes',
        'longitudes',
        'distinctLatitudes',
        'distinctLongitudes',
        'average',
        'standardDeviation',
        'kurtosis',
        'skewness',
        'referenceValue',
        'md5Headers',
        'md5Product',
        'md5Section1',
        'md5Section2',
        'md5Section3',
        'md5Section4',
        'md5Section5',
        'md5Section6',
        'md5Section7',
        'md5Section8',
        'zero',
        'sectionNumber',
        'numberOfSection',
        'codedValues',
        'pv',
        'maximum',
        'minimum',
        "parameterUnits"
    ]

def list_grib_files_tree(path,param1,param2,param3):
    """Function to list all GRIB files of a tree directory
    EFAS and ECV files wont be listed, change defparam 1 and 2 to work with these data sets
    param1 and param2 must be strings and in lowercase
    """
    defparam1 = "efas"
    defparam2 = "ecv"

    grib_files = []
    
    for root, dirs, files in os.walk(path):
            for f in files:
                if (f.endswith("grib") or f.endswith("grb")) and defparam1 not in f and defparam2 not in f:
                    grib_files.append(os.path.join(root,f))

    if param1 != '':
        grib_files = [item for item in grib_files if param1.lower() in item.lower()]
    
    if param2 != '':
        grib_files = [item for item in grib_files if param2.lower() in item.lower()]

    if param3 != '':
        grib_files = [item for item in grib_files if param3.lower() in item.lower()]
    
    grib_files.sort()
    
    return grib_files

def list_grib_files_folder(path,param1='',param2='',param3=''):
    """Function to list all GRIB files of a folder
    EFAS and ECV files wont be listed, change defparam 1 and 2 to work with these data sets
    param1 and param2 must be strings and in lowercase
    """
    defparam1 = "efas"
    defparam2 = "ecv"

    grib_files = []
    
    for f in os.listdir(path):
        if (f.endswith("grib") or f.endswith("grb")) and defparam1 not in f and defparam2 not in f:
            grib_files.append(os.path.join(path,f))
    
    if param1 != '':
        grib_files = [item for item in grib_files if param1.lower() in item.lower()]
    
    if param2 != '':
        grib_files = [item for item in grib_files if param2.lower() in item.lower()]

    if param3 != '':
        grib_files = [item for item in grib_files if param3.lower() in item.lower()]
    
    grib_files.sort()
    
    return grib_files

def map_keys(files):
    
    to_ignore = keys_to_ignore()
    
    vals_dict = {}
    vals_dict["maximum"] = {}
    vals_dict["minimum"] = {}
    vals_dict["parameterUnits"] = {}

    for f in files:

        with ecc.GribFile(f) as grib:
            for i in range(len(grib)):
                mssg = ecc.GribMessage(grib)
                try:
                    mssg_keys = mssg.keys()
                    var = mssg['cfVarName']

                    item = "maximum"
                    value_item = mssg[item]
                    if var not in vals_dict[item]:
                        vals_dict[item][var] = value_item
                    else:
                        if vals_dict[item][var] < value_item:
                            vals_dict[item][var] = value_item

                    item = "minimum"
                    value_item = mssg[item]
                    if var not in vals_dict[item]:
                        vals_dict[item][var] = value_item

                    item = "parameterUnits"
                    value_item = mssg[item]
                    if var not in vals_dict[item]:
                        vals_dict[item][var] = [value_item]
                        
                    else:
                        if value_item not in vals_dict[item][var]:
                            vals_dict[item][var].append(value_item)

                    for item in mssg_keys:
                        value_item = mssg[str(item)]
                        if item not in to_ignore:
                            if item not in vals_dict:
                                vals_dict[item] = [value_item]
                            if item in vals_dict:                
                                if not any(value_item==x for x in vals_dict[item]):
                                    vals_dict[item].append(value_item)

                except:
                    continue
                        
    return vals_dict

def save_dict2json(dic2save,param1='',param2='',param3=''):
    
    folder = os.path.join( os.getcwd(), 'VALUES_MAPPED')
    
    if not os.path.exists(folder):
        os.mkdir(folder)
    
    name = 'values_and_keys'

    if param1 != '' and param2 != '' and param3 != '':
        name += '-' + param1 + '-' + param2 + '-' + param3

    elif param1 != '' and param2 != '' and param3 == '':
        name += '-' + param1 + '-' + param2

    elif param1 != '' and param2 != '' and param3 == '':
        name += '-' + param1

    elif param1 == '' and param2 == '' and param3 == '':
        name += '.json'
    
    json_name = name = os.path.join(folder,name)
    
    with open(json_name,"w") as fp:
        json.dump(dic2save,fp,indent=4,sort_keys=True)

    fp.close()

    string = "\n#INFO | SAVED as: \n#\t" + name + "\n#INFO | In the directory: \n#\t" + folder + "\n" 

    return string

def print_help():
    string = "\n#####################################################\n"
    string += "# HELP for values_grib_keys.py script\n#\n"
    string += "# To run the script execute it like this:\n"
    string += "# $ python values_grib_keys.py decission directory field1 field2 field3\n#\n"
    string += "# decission = tree or folder\n#\n"
    string += "# directory = the directory you want to map\n#\n"
    string += "# field1 = seasonal or reanalysis\n"
    string += "# field2 = daily or monthly (for seasonal) era5 or uerra (for reanalysis)\n#\n"
    string += "# field3 = daily or monthly (for seasonal) era5 or uerra (for reanalysis)\n#\n#\n"
    string += "#####################################################\n"

    print(string)

def main(args):

    param1 = ''
    param2 = ''
    param3 = ''

    if len(args) == 2:
        decission = args[0]
        directory = args[1]
    if len(args) <= 5:
        decission = args[0]
        directory = args[1]
        try:
            param1 = args[2]
            param2 = args[3]
            param3 = args[3]
        except:
            pass

    if decission == "tree":
        # List all available GRIB file in the tree directory
        grib_files = list_grib_files_tree(directory,param1,param2,param3)

    elif decission == "folder":
        # List all available GRIB file in the folder
        grib_files = list_grib_files_folder(directory,param1,param2,param3)

    dict_vals = map_keys(grib_files)

    print(save_dict2json(dict_vals,param1,param2,param3))
        
if __name__ == "__main__":
    args = sys.argv[1:]

    if args == []:
        sys.exit(print_help())

    else:

        if args[0] in ["--help","-h"]:
            sys.exit(print_help())

        else:
            print("\nMapping all GRIB files in %s \n\n\t Product %s" %(args[1],args[2]))
            sys.exit(main(args))
